import numpy as np
from scipy import stats
from ..base_indicator import BaseIndicator
from indicators.common.common import CommonMethods

class VSA(BaseIndicator):
    def __init__(self):
        super().__init__("Volume Spread Analysis")

    def calculate(self, norm_lookback, open, high, low, close, volume):
        """
        Calculate Volume Spread Analysis
        """
        # Convert all inputs to numpy arrays and get their lengths
        open_prices = np.array(open)
        high_prices = np.array(high)
        low_prices = np.array(low)
        close_prices = np.array(close)
        volume = np.array(volume)
        
        n = len(close_prices)
        
        # Calculate ATR
        atr = CommonMethods.atr(open_prices, high_prices, low_prices, close_prices, norm_lookback)
        
        # Make sure ATR is the same length as other arrays by padding with NaN
        if len(atr) < n:
            pad_length = n - len(atr)
            atr = np.pad(atr, (pad_length, 0), mode='constant', constant_values=np.nan)
        
        # Calculate volume median using rolling window
        vol_med = np.zeros(n)
        for i in range(norm_lookback - 1, n):
            vol_med[i] = np.median(volume[max(0, i - norm_lookback + 1):i + 1])
        
        # Calculate normalized range and volume, handling potential NaN values
        norm_range = np.zeros(n)
        norm_volume = np.zeros(n)
        
        mask = (atr != 0) & ~np.isnan(atr)  # Avoid division by zero
        norm_range[mask] = (high_prices[mask] - low_prices[mask]) / atr[mask]
        
        mask = (vol_med != 0) & ~np.isnan(vol_med)  # Avoid division by zero
        norm_volume[mask] = volume[mask] / vol_med[mask]
        
        # Initialize range deviation array
        range_dev = np.full(n, np.nan)
        
        # Calculate range deviation using rolling regression
        for i in range(norm_lookback * 2, n):
            # Get window of data
            window_vol = norm_volume[i - norm_lookback + 1:i + 1]
            window_range = norm_range[i - norm_lookback + 1:i + 1]
            
            # Remove any NaN values for regression
            mask = ~np.isnan(window_vol) & ~np.isnan(window_range)
            if not np.any(mask):
                continue
                
            window_vol = window_vol[mask]
            window_range = window_range[mask]
            
            # Perform linear regression
            if len(window_vol) > 1:  # Need at least 2 points for regression
                slope, intercept, r_val, _, _ = stats.linregress(window_vol, window_range)
                
                if slope <= 0.0 or r_val < 0.2:
                    range_dev[i] = 0.0
                    continue
                
                pred_range = intercept + slope * norm_volume[i]
                range_dev[i] = norm_range[i] - pred_range
            
        self.result = range_dev
        return range_dev